@@ -1,16 +1,19 @@
 import os
 
-SF_MIRROR = "http://iweb.dl.sourceforge.net"
-PILLOW_DEPENDS_DIR = "C:\\pillow-depends\\"
+SF_MIRROR = "https://iweb.dl.sourceforge.net"
 
 pythons = {
-    "27": {"compiler": 7, "vc": 2010},
-    "pypy2": {"compiler": 7, "vc": 2010},
+    "pypy3": {"compiler": 7.1, "vc": 2015},
+    # for AppVeyor
     "35": {"compiler": 7.1, "vc": 2015},
     "36": {"compiler": 7.1, "vc": 2015},
-    "pypy3": {"compiler": 7.1, "vc": 2015},
     "37": {"compiler": 7.1, "vc": 2015},
     "38": {"compiler": 7.1, "vc": 2015},
+    # for GitHub Actions
+    "3.5": {"compiler": 7.1, "vc": 2015},
+    "3.6": {"compiler": 7.1, "vc": 2015},
+    "3.7": {"compiler": 7.1, "vc": 2015},
+    "3.8": {"compiler": 7.1, "vc": 2015},
 }
 
 VIRT_BASE = "c:/vp/"
@@ -23,66 +26,92 @@ libs = {
     # },
     "zlib": {
         "url": "http://zlib.net/zlib1211.zip",
-        "filename": PILLOW_DEPENDS_DIR + "zlib1211.zip",
+        "filename": "zlib1211.zip",
         "dir": "zlib-1.2.11",
     },
     "jpeg": {
-        "url": "http://www.ijg.org/files/jpegsr9c.zip",
-        "filename": PILLOW_DEPENDS_DIR + "jpegsr9c.zip",
-        "dir": "jpeg-9c",
+        "url": "http://www.ijg.org/files/jpegsr9d.zip",
+        "filename": "jpegsr9d.zip",
+        "dir": "jpeg-9d",
     },
     "tiff": {
-        "url": "ftp://download.osgeo.org/libtiff/tiff-4.0.10.tar.gz",
-        "filename": PILLOW_DEPENDS_DIR + "tiff-4.0.10.tar.gz",
-        "dir": "tiff-4.0.10",
+        "url": "ftp://download.osgeo.org/libtiff/tiff-4.1.0.tar.gz",
+        "filename": "tiff-4.1.0.tar.gz",
+        "dir": "tiff-4.1.0",
     },
     "freetype": {
         "url": "https://download.savannah.gnu.org/releases/freetype/freetype-2.10.1.tar.gz",  # noqa: E501
-        "filename": PILLOW_DEPENDS_DIR + "freetype-2.10.1.tar.gz",
+        "filename": "freetype-2.10.1.tar.gz",
         "dir": "freetype-2.10.1",
     },
-    "lcms": {
+    "lcms-2.7": {
         "url": SF_MIRROR + "/project/lcms/lcms/2.7/lcms2-2.7.zip",
-        "filename": PILLOW_DEPENDS_DIR + "lcms2-2.7.zip",
+        "filename": "lcms2-2.7.zip",
         "dir": "lcms2-2.7",
     },
-    "ghostscript": {
-        "url": "https://github.com/ArtifexSoftware/ghostpdl-downloads/releases/download/gs927/ghostscript-9.27.tar.gz",  # noqa: E501
-        "filename": PILLOW_DEPENDS_DIR + "ghostscript-9.27.tar.gz",
-        "dir": "ghostscript-9.27",
+    "lcms-2.8": {
+        "url": SF_MIRROR + "/project/lcms/lcms/2.8/lcms2-2.8.zip",
+        "filename": "lcms2-2.8.zip",
+        "dir": "lcms2-2.8",
     },
     "tcl-8.5": {
         "url": SF_MIRROR + "/project/tcl/Tcl/8.5.19/tcl8519-src.zip",
-        "filename": PILLOW_DEPENDS_DIR + "tcl8519-src.zip",
+        "filename": "tcl8519-src.zip",
         "dir": "",
     },
     "tk-8.5": {
         "url": SF_MIRROR + "/project/tcl/Tcl/8.5.19/tk8519-src.zip",
-        "filename": PILLOW_DEPENDS_DIR + "tk8519-src.zip",
+        "filename": "tk8519-src.zip",
         "dir": "",
         "version": "8.5.19",
     },
     "tcl-8.6": {
-        "url": SF_MIRROR + "/project/tcl/Tcl/8.6.9/tcl869-src.zip",
-        "filename": PILLOW_DEPENDS_DIR + "tcl869-src.zip",
+        "url": SF_MIRROR + "/project/tcl/Tcl/8.6.10/tcl8610-src.zip",
+        "filename": "tcl8610-src.zip",
         "dir": "",
     },
     "tk-8.6": {
-        "url": SF_MIRROR + "/project/tcl/Tcl/8.6.9/tk869-src.zip",
-        "filename": PILLOW_DEPENDS_DIR + "tk869-src.zip",
+        "url": SF_MIRROR + "/project/tcl/Tcl/8.6.10/tk8610-src.zip",
+        "filename": "tk8610-src.zip",
         "dir": "",
-        "version": "8.6.9",
+        "version": "8.6.10",
     },
     "webp": {
-        "url": "http://downloads.webmproject.org/releases/webp/libwebp-1.0.3.tar.gz",
-        "filename": PILLOW_DEPENDS_DIR + "libwebp-1.0.3.tar.gz",
-        "dir": "libwebp-1.0.3",
+        "url": "http://downloads.webmproject.org/releases/webp/libwebp-1.1.0.tar.gz",
+        "filename": "libwebp-1.1.0.tar.gz",
+        "dir": "libwebp-1.1.0",
     },
     "openjpeg": {
         "url": "https://github.com/uclouvain/openjpeg/archive/v2.3.1.tar.gz",
-        "filename": PILLOW_DEPENDS_DIR + "openjpeg-2.3.1.tar.gz",
+        "filename": "openjpeg-2.3.1.tar.gz",
         "dir": "openjpeg-2.3.1",
     },
+    "jpeg-turbo": {
+        "url": SF_MIRROR + "/project/libjpeg-turbo/2.0.3/libjpeg-turbo-2.0.3.tar.gz",
+        "filename": "libjpeg-turbo-2.0.3.tar.gz",
+        "dir": "libjpeg-turbo-2.0.3",
+    },
+    # e5d454b: Merge tag '2.12.6' into msvc
+    "imagequant": {
+        "url": "https://github.com/ImageOptim/libimagequant/archive/e5d454bc7f5eb63ee50c84a83a7fa5ac94f68ec4.zip",  # noqa: E501
+        "filename": "libimagequant-e5d454bc7f5eb63ee50c84a83a7fa5ac94f68ec4.zip",
+        "dir": "libimagequant-e5d454bc7f5eb63ee50c84a83a7fa5ac94f68ec4",
+    },
+    "harfbuzz": {
+        "url": "https://github.com/harfbuzz/harfbuzz/archive/2.6.4.zip",
+        "filename": "harfbuzz-2.6.4.zip",
+        "dir": "harfbuzz-2.6.4",
+    },
+    "fribidi": {
+        "url": "https://github.com/fribidi/fribidi/archive/v1.0.9.zip",
+        "filename": "fribidi-1.0.9.zip",
+        "dir": "fribidi-1.0.9",
+    },
+    "libraqm": {
+        "url": "https://github.com/HOST-Oman/libraqm/archive/v0.7.0.zip",
+        "filename": "libraqm-0.7.0.zip",
+        "dir": "libraqm-0.7.0",
+    },
 }
 
 compilers = {
@@ -132,14 +161,14 @@ compilers = {
 def pyversion_from_env():
     py = os.environ["PYTHON"]
 
-    py_version = "27"
+    py_version = "35"
     for k in pythons:
         if k in py:
             py_version = k
             break
 
     if "64" in py:
-        py_version = "%s%s" % (py_version, X64_EXT)
+        py_version = "{}{}".format(py_version, X64_EXT)
 
     return py_version
 
